var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw Error('Dynamic require of "' + x + '" is not supported');
});

// src/lib/getTSConfig.ts
import { readFileSync } from "fs";
import merge from "lodash.merge";
import { dirname, resolve } from "path";
import ts from "typescript";
function getTSConfig(opts) {
  var _a, _b;
  let configPath = (_b = opts.configPath) != null ? _b : ts.findConfigFile(
    (_a = opts.searchPath) != null ? _a : process.cwd(),
    ts.sys.fileExists,
    opts.configName
  );
  if (!configPath) {
    throw new Error("No config file found");
  }
  if (configPath.startsWith(".")) {
    configPath = __require.resolve(configPath);
  }
  const config = ts.readConfigFile(
    configPath,
    (path) => readFileSync(path, "utf-8")
  );
  if (config.config.extends) {
    const parentConfig = getTSConfig({
      ...opts,
      configPath: resolve(dirname(configPath), config.config.extends)
    });
    config.config = merge(config.config, parentConfig);
  }
  if (config.error) {
    throw config.error;
  } else {
    return config.config;
  }
}

// src/lib/humanizeFileSize.ts
function humanizeFileSize(size) {
  const i = Math.floor(Math.log(size) / Math.log(1024));
  return Math.round(size / Math.pow(1024, i) * 100) / 100 + ["b", "kb", "mb", "gb", "tb"][i];
}

// src/plugin.ts
import chalk from "chalk";
import { existsSync, lstatSync } from "fs";
import { resolve as resolve3 } from "path";
import ts3 from "typescript";

// src/lib/getCompilerOptions.ts
import { createHash } from "crypto";
import { resolve as resolve2 } from "path";
import { tmpdir } from "tmp";
import ts2 from "typescript";
function getCompilerOptions(opts) {
  var _a, _b, _c, _d;
  const compilerOptions = ts2.convertCompilerOptionsFromJson(
    opts.tsconfig.compilerOptions,
    process.cwd()
  ).options;
  compilerOptions.declaration = true;
  compilerOptions.emitDeclarationOnly = true;
  if (!compilerOptions.declarationDir) {
    compilerOptions.declarationDir = (_b = (_a = compilerOptions.declarationDir) != null ? _a : opts.esbuildOptions.outdir) != null ? _b : compilerOptions.outDir;
  }
  if (compilerOptions.incremental && !compilerOptions.tsBuildInfoFile) {
    const configHash = createHash("sha256").update(
      JSON.stringify({
        compilerOptions,
        __buildContext: (_c = opts.pluginOptions) == null ? void 0 : _c.__buildContext
      })
    ).digest("hex");
    compilerOptions.tsBuildInfoFile = resolve2(
      (_d = opts.pluginOptions.buildInfoDir) != null ? _d : tmpdir,
      `esbuild-plugin-dts-${configHash}.tsbuildinfo`
    );
  }
  compilerOptions.listEmittedFiles = true;
  return compilerOptions;
}

// src/lib/logger.ts
function createLogger(logLevel) {
  const levels = [
    "verbose",
    "debug",
    "info",
    "warning",
    "error",
    "silent"
  ];
  for (const l of levels) {
    if (l === logLevel) {
      break;
    } else {
      levels.splice(levels.indexOf(l), 1);
    }
  }
  return {
    info: (...msg) => {
      if (levels.includes("info"))
        console.log(...msg);
    }
  };
}

// src/plugin.ts
var dtsPlugin = (opts = {}) => ({
  name: "dts-plugin",
  async setup(build) {
    const log = createLogger(build.initialOptions.logLevel);
    const config = opts.tsconfig && typeof opts.tsconfig !== "string" ? opts.tsconfig : getTSConfig({
      configPath: opts.tsconfig
    });
    const compilerOptions = getCompilerOptions({
      tsconfig: config,
      pluginOptions: opts,
      esbuildOptions: build.initialOptions
    });
    const compilerHost = compilerOptions.incremental ? ts3.createIncrementalCompilerHost(compilerOptions) : ts3.createCompilerHost(compilerOptions);
    const inputFiles = [];
    build.onLoad({ filter: /(\.tsx|\.ts)$/ }, async (args) => {
      var _a;
      inputFiles.push(args.path);
      compilerHost.getSourceFile(
        args.path,
        (_a = compilerOptions.target) != null ? _a : ts3.ScriptTarget.Latest,
        (m) => console.log(m),
        true
      );
      return {};
    });
    build.onEnd(() => {
      let compilerProgram;
      if (compilerOptions.incremental) {
        compilerProgram = ts3.createIncrementalProgram({
          options: compilerOptions,
          host: compilerHost,
          rootNames: inputFiles
        });
      } else {
        compilerProgram = ts3.createProgram(
          inputFiles,
          compilerOptions,
          compilerHost
        );
      }
      const startTime = Date.now();
      const emitResult = compilerProgram.emit();
      if (emitResult.emitSkipped || typeof emitResult.emittedFiles === "undefined") {
        log.info(chalk`{yellow No declarations emitted}`);
      } else {
        for (const emittedFile of emitResult.emittedFiles) {
          const emittedPath = resolve3(emittedFile);
          if (existsSync(emittedPath) && emittedPath !== compilerOptions.tsBuildInfoFile) {
            const stat = lstatSync(emittedPath);
            const pathFromContentRoot = emittedPath.replace(resolve3(process.cwd()), "").replace(/^[\\/]/, "");
            const humanFileSize = humanizeFileSize(stat.size);
            log.info(
              chalk`  {bold ${pathFromContentRoot}} {cyan ${humanFileSize}}`
            );
          }
        }
      }
      log.info(
        chalk`{green Finished compiling declarations in ${Date.now() - startTime}ms}`
      );
    });
  }
});
export {
  dtsPlugin as default,
  dtsPlugin,
  getTSConfig,
  humanizeFileSize
};
//# sourceMappingURL=index.mjs.map